# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets


class Widget(QtWidgets.QWidget):

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = Widget()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore

H_ALIGNMENTS = dict(left=QtCore.Qt.AlignLeft,
                    right=QtCore.Qt.AlignRight,
                    center=QtCore.Qt.AlignHCenter,
                    justify=QtCore.Qt.AlignJustify)

V_ALIGNMENTS = dict(top=QtCore.Qt.AlignTop,
                    bottom=QtCore.Qt.AlignBottom,
                    center=QtCore.Qt.AlignVCenter,
                    baseline=QtCore.Qt.AlignBaseline)

TEXT_INTERACTION = dict(none=QtCore.Qt.NoTextInteraction,
                        by_mouse=QtCore.Qt.NoTextInteraction,
                        by_keyboard=QtCore.Qt.NoTextInteraction)


class Label(QtWidgets.QLabel):

    def set_alignment(self, horizontal=None, vertical=None):
        if horizontal is None and vertical is not None:
            flag = V_ALIGNMENTS.get(vertical)
        elif vertical is None and horizontal is not None:
            flag = H_ALIGNMENTS.get(horizontal)
        elif vertical is not None and horizontal is not None:
            flag = V_ALIGNMENTS.get(vertical) | H_ALIGNMENTS.get(horizontal)
        else:
            return
        self.setAlignment(flag)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = Label("test")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtGui


class TextEdit(QtWidgets.QTextEdit):

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def set_font(self,
                 font_name: str,
                 font_size: int = -1,
                 weight: int = -1,
                 italic: bool = False):
        font = QtGui.QFont(font_name, font_size, weight, italic)
        self.setFont(font)

    def append(self, text: str):
        self.appendPlainText(text)

    def set_text(self, text: str):
        self.setPlainText(text)

    def text(self) -> str:
        return self.toPlainText()

    def set_read_only(self, value: bool):
        self.setReadOnly(value)

    def scroll_to_end(self):
        """scroll to the end of the text
        """
        self.verticalScrollBar().setValue(self.verticalScrollBar().maximum())


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = TextEdit("This is a test")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore
import qtawesome as qta


class Application(QtWidgets.QApplication):

    def set_icon(self, icon):
        if icon:
            if isinstance(icon, str):
                icon = qta.icon(icon, color="lightgray")
            self.setWindowIcon(icon)

    def load_language_file(self, path):
        translator = QtCore.QTranslator(self)
        translator.load(str(path))
        self.installTranslator(translator)

    def set_metadata(self, app_name=None, org_name=None, org_domain=None):
        if app_name:
            self.setApplicationName(app_name)
        if org_name:
            self.setOrganizationName(org_name)
        if org_domain:
            self.setOrganizationDomain(org_domain)

    @classmethod
    def use_hdpi_bitmaps(cls):
        QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)

    @classmethod
    def disable_window_help_button(cls):
        QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_DisableWindowContextHelpButton)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""


from qtpy import QtWidgets, QtCore


class DockWidget(QtWidgets.QDockWidget):
    """
    Customized DockWidget class
    contains a custom TitleBar with maximise button
    """

    def __init__(self, *args, **kwargs):
        name = kwargs.pop("name", None)
        title = kwargs.pop("title", None)
        super().__init__(*args, **kwargs)
        if name:
            self.setObjectName(name)
        if title:
            self.setWindowTitle(title)
        self.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)

    def setup_title_bar(self):
        title_bar = QtWidgets.QWidget()
        layout = QtWidgets.QHBoxLayout()
        layout.setAlignment(QtCore.Qt.AlignRight)
        title_bar.setLayout(layout)
        maximise_button = QtWidgets.QPushButton()
        layout.addWidget(maximise_button)
        style = QtWidgets.QStyle.SP_TitleBarMaxButton
        icon = maximise_button.style().standardIcon(style, None, maximise_button)
        maximise_button.setIcon(icon)
        maximise_button.clicked.connect(self.maximise)
        close_button = QtWidgets.QPushButton()
        layout.addWidget(close_button)
        style = QtWidgets.QStyle.SP_TitleBarCloseButton
        icon = close_button.style().standardIcon(style, None, close_button)
        close_button.setIcon(icon)
        close_button.clicked.connect(self.close)
        # self.setTitleBarWidget(title_bar)

    def maximise(self):
        if not self.isFloating():
            self.setFloating(True)
        self.showMaximized()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets


class MenuBar(QtWidgets.QMenuBar):

    def add_action(self, action):
        self.addAction(action)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    win = QtWidgets.QMainWindow()
    menu_bar = MenuBar()
    win.setMenuBar(menu_bar)
    win.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Optional, Callable, Iterable

from qtpy import QtCore, QtWidgets


class HeaderView(QtWidgets.QHeaderView):
    MODES = dict(interactive=QtWidgets.QHeaderView.Interactive,
                 fixed=QtWidgets.QHeaderView.Fixed,
                 stretch=QtWidgets.QHeaderView.Stretch,
                 resize_to_contents=QtWidgets.QHeaderView.ResizeToContents)

    section_vis_changed = QtCore.Signal(int, bool)

    def __init__(self, parent):
        super().__init__(QtCore.Qt.Horizontal, parent=parent)
        self.setSectionsMovable(True)
        self.setSectionsClickable(True)

    def resize_sections(self, mode: str):
        self.resizeSections(self.MODES[mode])

    def resize_mode(self, mode: str, col: Optional[int] = None):
        if mode not in self.MODES:
            raise ValueError("mode not existing")
        if col is None:
            self.setSectionResizeMode(self.MODES[mode])
        else:
            self.setSectionResizeMode(col, self.MODES[mode])

    def section_labels(self):
        model = self.parent().model()
        return [model.headerData(i, QtCore.Qt.Horizontal, QtCore.Qt.DisplayRole)
                for i in range(self.count())]

    def contextMenuEvent(self, event):
        """
        context menu for our files tree
        """
        menu = QtWidgets.QMenu(parent=self)
        for i, header_label in enumerate(self.section_labels()[1:], start=1):
            act = menu.addAction(header_label)
            act.setCheckable(True)
            val = not self.isSectionHidden(i)
            act.setChecked(val)
            self.setSectionHidden(i, val)
            self.section_vis_changed.emit(i, val)
        menu.exec_(self.mapToGlobal(event.pos()))

    def set_custom_menu(self, method: Callable):
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(method)

    def set_sizes(self, sizes: Iterable):
        for i, size in enumerate(sizes):
            if size is not None:
                self.resizeSection(i, size)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import pathlib

from qtpy import QtGui, QtWidgets, QtCore
from prettyqt import widgets


H_ALIGNMENTS = dict(left=QtCore.Qt.AlignLeft,
                    right=QtCore.Qt.AlignRight,
                    center=QtCore.Qt.AlignHCenter,
                    justify=QtCore.Qt.AlignJustify)

V_ALIGNMENTS = dict(top=QtCore.Qt.AlignTop,
                    bottom=QtCore.Qt.AlignBottom,
                    center=QtCore.Qt.AlignVCenter,
                    baseline=QtCore.Qt.AlignBaseline)


class Image(widgets.Label):

    def __init__(self, path, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setScaledContents(True)
        self.set_alignment(horizontal="center")
        self.set_image(path, width=300)

    def set_image(self, path, width=300):
        self.setText("<html><head/><body><p>"
                     f'<img src="{path}" width="{width}"/>'
                     "</p></body></html>")

    @classmethod
    def from_path(cls, path: pathlib.Path, parent=None) -> "Image":
        with path.open(mode="rb") as f:
            data = f.read()
        # Create widget
        pixmap = QtGui.QPixmap()
        pixmap.loadFromData(data)
        label = widgets.Label(parent=parent)
        label.setPixmap(pixmap)
        label.resize(pixmap.width(), pixmap.height())
        return label


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = Image(path="")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore


INSERT_POLICIES = dict(no_insert=QtWidgets.QComboBox.NoInsert,
                       top=QtWidgets.QComboBox.InsertAtTop,
                       current=QtWidgets.QComboBox.InsertAtCurrent,
                       bottom=QtWidgets.QComboBox.InsertAtBottom,
                       after_current=QtWidgets.QComboBox.InsertAfterCurrent,
                       before_current=QtWidgets.QComboBox.InsertBeforeCurrent,
                       alphabetically=QtWidgets.QComboBox.InsertAlphabetically)

SIZE_POLICIES = dict(content=QtWidgets.QComboBox.AdjustToContents,
                     first_show=QtWidgets.QComboBox.AdjustToContentsOnFirstShow,
                     min_length=QtWidgets.QComboBox.AdjustToMinimumContentsLength,
                     min_length_with_icon=QtWidgets.QComboBox.AdjustToMinimumContentsLengthWithIcon)


class ComboBox(QtWidgets.QComboBox):

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def add_item(self, label, data=None, icon=None):
        if icon is not None:
            self.addItem(icon, label, userData=data)
        else:
            self.addItem(label, userData=data)

    def set_insert_policy(self, policy):
        if policy not in INSERT_POLICIES:
            raise ValueError("Policy not available")
        policy = INSERT_POLICIES.get(policy)
        self.setInsertPolicy(policy)

    def set_size_policy(self, policy):
        """set size policy

        Args:
            policy: size policy to use

        Raises:
            ValueError: invalid size policy
        """
        if policy not in SIZE_POLICIES:
            raise ValueError("Policy not available")
        policy = SIZE_POLICIES.get(policy)
        self.setSizeAdjustPolicy(policy)

    def set_icon_size(self, size: int):
        self.setIconSize(QtCore.QSize(size, size))

    def set_min_char_length(self, chars):
        self.setMinimumContentsLength(chars)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = ComboBox()
    widget.add_item("test", data="aa")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets
from prettyqt import widgets


MODES = dict(maximum=QtWidgets.QLayout.SetMaximumSize,
             fixed=QtWidgets.QLayout.SetFixedSize)


class FormLayout(QtWidgets.QFormLayout):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_size_mode("maximum")
        self.setVerticalSpacing(8)

    def __getitem__(self, index):
        return self.itemAt(index)

    def set_size_mode(self, mode: str):
        if mode not in MODES:
            raise ValueError(f"{mode} not a valid size mode.")
        self.setSizeConstraint(MODES[mode])

    def set_label_widget(self, row: int, widget):
        """set a widget for the label position at given row

        Args:
            row: Row offset
            widget: widget to get added to layout
        """
        if isinstance(widget, str):
            widget = widgets.Label(widget)
        self.setWidget(row, QtWidgets.QFormLayout.LabelRole, widget)

    def set_field_widget(self, row: int, widget):
        """set a widget for the field position at given row

        Args:
            row: Row offset
            widget: widget to get added to layout
        """
        if isinstance(widget, str):
            widget = widgets.Label(widget)
        self.setWidget(row, QtWidgets.QFormLayout.FieldRole, widget)

    def set_spanning_widget(self, row: int, widget):
        """set a widget spanning label and field position at given row

        Args:
            row: Row offset
            widget: widget to get added to layout
        """
        if isinstance(widget, str):
            widget = widgets.Label(widget)
        self.setWidget(row, QtWidgets.QFormLayout.SpanningRole, widget)

    @classmethod
    def from_dict(cls, dct, parent=None):
        formlayout = FormLayout(parent)
        for i, (k, v) in enumerate(dct.items(), start=1):
            if k is not None:
                formlayout.set_label_widget(i, k)
            if v is not None:
                formlayout.set_field_widget(i, v)
        return formlayout


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    dct = {"key": widgets.Label("test"),
           None: widgets.Label("test 2")}
    layout = FormLayout.from_dict(dct)
    widget = QtWidgets.QWidget()
    widget.setLayout(layout)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Callable

from qtpy import QtWidgets, QtCore
import qtawesome as qta

STYLES = {"icon": QtCore.Qt.ToolButtonIconOnly,
          "text": QtCore.Qt.ToolButtonTextOnly,
          "text_beside_icon": QtCore.Qt.ToolButtonTextBesideIcon,
          "text_below_icon": QtCore.Qt.ToolButtonTextUnderIcon}


class Toolbar(QtWidgets.QToolBar):
    """
    Customized Toolbar class
    """

    ICON_SIZE = 20

    def __init__(self, *args, **kwargs):
        self.menu_buttons = list()
        super().__init__(*args, **kwargs)
        self.set_icon_size(self.ICON_SIZE)

    def add_menu_button(self,
                        label: str,
                        icon,
                        menu: QtWidgets.QMenu,
                        style: str = None) -> QtWidgets.QToolButton:
        btn = QtWidgets.QToolButton()
        btn.setText(label)
        if style:
            btn.setToolButtonStyle(STYLES[style])
        btn.setMenu(menu)
        if isinstance(icon, str):
            icon = qta.icon(icon)
        btn.setIcon(icon)
        btn.setPopupMode(QtWidgets.QToolButton.InstantPopup)
        self.menu_buttons.append(btn)
        self.addWidget(btn)
        return btn

    def set_style(self, style):
        if style is None:
            return None
        self.setToolButtonStyle(style)
        for btn in self.menu_buttons:
            self.setToolButtonStyle(style)

    def add_action(self, label: str, icon, callback: Callable, checkable=False):
        if isinstance(icon, str):
            icon = qta.icon(icon)
        action = self.addAction(icon, label, callback)
        if checkable:
            action.setCheckable(True)
        return action

    def set_icon_size(self, size: int):
        self.setIconSize(QtCore.QSize(size, size))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = Toolbar()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

"""widgets module

contains QtWidgets-based classes
"""

from .application import Application

from .widget import Widget
from .dialog import Dialog

from .formlayout import FormLayout

from .slider import Slider
from .styleoptionslider import StyleOptionSlider

from .action import Action
from .menu import Menu
from .menubar import MenuBar
from .statusbar import StatusBar
from .tabbar import TabBar
from .tabwidget import TabWidget
from .toolbar import Toolbar
from .headerview import HeaderView
from .dockwidget import DockWidget
from .label import Label
from .pushbutton import PushButton
from .radiobutton import RadioButton
from .combobox import ComboBox
from .lineedit import LineEdit
from .textedit import TextEdit
from .plaintextedit import PlainTextEdit
from .textbrowser import TextBrowser

from .splashscreen import SplashScreen
from .progressdialog import ProgressDialog
from .filedialog import FileDialog
from .dialogbuttonbox import DialogButtonBox

from .mainwindow import MainWindow

from .composed.spanslider import SpanSlider
from .composed.waitingspinner import WaitingSpinner
from .composed.callout import Callout
from .composed.markdownwidget import MarkdownWindow, MarkdownWidget
from .composed.image import Image
from .composed.imageviewer import ImageViewer
from .composed.popupinfo import PopupInfo
from .composed.buttondelegate import ButtonDelegate

__all__ = ["Application",
           "Widget",
           "Dialog",
           "FormLayout",
           "Slider",
           "StyleOptionSlider",
           "Action",
           "Menu",
           "MenuBar",
           "StatusBar",
           "TabWidget",
           "TabBar",
           "Toolbar",
           "HeaderView",
           "DockWidget",
           "Label",
           "PushButton",
           "RadioButton",
           "ComboBox",
           "LineEdit",
           "TextEdit",
           "PlainTextEdit",
           "TextBrowser",
           "SplashScreen",
           "ProgressDialog",
           "FileDialog",
           "DialogButtonBox",
           "MainWindow",
           "SpanSlider",
           "WaitingSpinner",
           "Callout",
           "PopupInfo",
           "ButtonDelegate",
           "Image",
           "ImageViewer",
           "MarkdownWindow",
           "MarkdownWidget"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets


class StatusBar(QtWidgets.QStatusBar):
    """
    Customized StatusBar class
    """

    def add_action(self, action):
        self.addAction(action)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    dlg = QtWidgets.QMainWindow()
    status_bar = StatusBar()
    dlg.setStatusBar(status_bar)
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import sys
import pathlib
from qtpy import QtWidgets

from prettyqt import widgets


class ImageViewer(QtWidgets.QWidget):

    def __init__(self, title="", parent=None):
        super().__init__(parent)
        self.title = title
        self.left = 10
        self.top = 10
        self.width = 640
        self.height = 480
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)
        path = pathlib.Path("decisiontree.png")
        self.image = widgets.Image.from_path(path, parent=self)
        self.show()


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ex = ImageViewer()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

"""Top-level package for PrettyQt."""

from . import core
from . import widgets

__author__ = """Philipp Temminghoff"""
__email__ = 'phil65@kodi.tv'
__version__ = '0.9.0'

__all__ = ["core", "widgets"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import pathlib

from qtpy import QtWidgets, QtCore

MODES = dict(existing_file=QtWidgets.QFileDialog.ExistingFile,
             existing_files=QtWidgets.QFileDialog.ExistingFiles,
             any_file=QtWidgets.QFileDialog.ExistingFiles,
             directory=QtWidgets.QFileDialog.ExistingFiles)

LABELS = dict(look_in=QtWidgets.QFileDialog.LookIn,
              filename=QtWidgets.QFileDialog.FileName,
              filetype=QtWidgets.QFileDialog.FileType,
              accept=QtWidgets.QFileDialog.Accept,
              reject=QtWidgets.QFileDialog.Reject)


class FileDialog(QtWidgets.QFileDialog):
    """
    simple dialog used to display some widget
    """

    def __init__(self, path=None, mode=None, caption="", path_id=None, parent=None):
        self.path_id = path_id
        settings = QtCore.QSettings()
        initial_path = settings.value(self.path_id, "")
        super().__init__(directory=initial_path, caption=caption, parent=parent)
        self.set_file_mode("existing_files")
        self.set_accept_mode(mode)

    def set_accept_mode(self, mode: str):
        if mode == "save":
            self.setAcceptMode(self.AcceptSave)
        else:
            self.setAcceptMode(self.AcceptOpen)

    def set_label_text(self, label):
        if label not in LABELS:
            raise ValueError(f"Invalid value. Valid values: {LABELS.keys()}")
        self.setLabelText(LABELS[label])

    def set_file_mode(self, mode: str):
        """sets the file mode of the dialog

        allowed values are 'existing_file', 'existing_files',
        'any_file' and 'directory'

        Args:
            mode: mode to use
        """
        self.setFileMode(MODES[mode])

    def selected_files(self):
        return [pathlib.Path(p) for p in self.selectedFiles()]

    def selected_file(self):
        selected = self.selectedFiles()
        if selected:
            return pathlib.Path(selected[0])

    def choose_folder(self):
        self.setFileMode(QtWidgets.QFileDialog.Directory)
        return self.choose()

    def open_file(self):
        self.setFileMode(QtWidgets.QFileDialog.ExistingFile)
        return self.choose()

    def choose(self):
        result = super().exec_()
        if result != QtWidgets.QDialog.Accepted:
            return None
        paths = self.selected_files()
        folder_path = paths[0].parent
        if self.path_id:
            settings = QtCore.QSettings()
            settings.setValue(self.path_id, str(folder_path))
        return paths

    def set_filter(self, extension_dict):
        """set filter based on given dictionary

        dict must contain "'name': ['.ext1', '.ext2']" as key-value pairs

        Args:
            extension_dict: filter dictionary
        """
        items = [f"{k} ({' '.join(f'*{ext}' for ext in v)})"
                 for k, v in extension_dict.items()]
        filter_str = ";;".join(items)
        self.setNameFilter(filter_str)

    def directory(self) -> pathlib.Path:
        """return current directory

        returns current directory level as a Pathlib object

        Returns:
            Pathlib object
        """
        return pathlib.Path(super().directory())


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = FileDialog(path_id="test", caption="Some header")
    widget.show()
    app.exec_()
